@@ -15,43 +15,46 @@
 
 There should be a file 'ftesting.zcml' in the current directory.
 
-$Id: functional.py 119446 2011-01-07 19:50:06Z mgedmin $
 """
+
 import copy
 import doctest
+import io
 import logging
 import os.path
 import re
-import rfc822
 import sys
 import traceback
 import unittest
-from StringIO import StringIO
-from Cookie import SimpleCookie
-from transaction import abort, commit
+from http.cookies import SimpleCookie
+
+import zope.app.appsetup.product
+from transaction import abort
+from transaction import commit
 from ZODB.DB import DB
 from ZODB.DemoStorage import DemoStorage
 from ZODB.interfaces import IDatabase
-
-from zope import component
-from zope.publisher.interfaces import ISkinnable
-from zope.publisher.browser import BrowserRequest
-from zope.publisher.http import HTTPRequest
-from zope.publisher.publish import publish
-from zope.publisher.skinnable import setDefaultSkin
-from zope.security.interfaces import Forbidden, Unauthorized
-
-import zope.app.appsetup.product
-import zope.app.testing.setup
 from zope.app.appsetup.appsetup import multi_database
 from zope.app.debug import Debugger
 from zope.app.publication.http import HTTPPublication
-from zope.app.publication.zopepublication import ZopePublication
 from zope.app.publication.httpfactory import chooseClasses
-from zope.component.hooks import setSite, getSite
+from zope.app.publication.zopepublication import ZopePublication
+from zope.component.hooks import getSite
+from zope.component.hooks import setSite
+from zope.publisher.browser import BrowserRequest
+from zope.publisher.http import HTTPRequest
+from zope.publisher.interfaces import ISkinnable
+from zope.publisher.publish import publish
+from zope.publisher.skinnable import setDefaultSkin
+from zope.security.interfaces import Forbidden
+from zope.security.interfaces import Unauthorized
+
+import zope.app.testing.setup
+from zope import component
+from zope.app.testing._compat import headers_factory
 
 
-class ResponseWrapper(object):
+class ResponseWrapper:
     """A wrapper that adds several introspective methods to a response."""
 
     def __init__(self, response, path, request, omit=()):
@@ -65,22 +68,39 @@ class ResponseWrapper(object):
         """Returns the full HTTP output (headers + body)"""
         body = self.getBody()
         omit = self.omit
-        headers = [x
-                   for x in self._response.getHeaders()
-                   if x[0].lower() not in omit]
-        headers.sort()
-        headers = '\n'.join([("%s: %s" % (n, v)) for (n, v) in headers])
-        statusline = '%s %s' % (self._response._request['SERVER_PROTOCOL'],
-                                self._response.getStatusString())
+        headers = sorted([x
+                          for x in self._response.getHeaders()
+                          if x[0].lower() not in omit])
+        headers = '\n'.join([("{}: {}".format(n, v)) for (n, v) in headers])
+        statusline = '{} {}'.format(self._response._request['SERVER_PROTOCOL'],
+                                    self._response.getStatusString())
         if body:
-            return '%s\n%s\n\n%s' % (statusline, headers, body)
+            return '{}\n{}\n\n{}'.format(statusline, headers, body)
         else:
-            return '%s\n%s\n' % (statusline, headers)
+            return '{}\n{}\n'.format(statusline, headers)
 
     def getBody(self):
         """Returns the response body"""
         if self._body is None:
-            self._body = ''.join(self._response.consumeBody())
+            try:
+                b = self._response.consumeBody()
+            except TypeError:
+                from zope.publisher.http import DirectResult
+                from zope.publisher.xmlrpc import XMLRPCResponse
+                if (isinstance(self._response, XMLRPCResponse)
+                        and isinstance(
+                            getattr(self._response, '_result', None),
+                            DirectResult)):
+                    # Somewhere in the publisher we're getting a DirectResult
+                    # whose '_result' body is a sequence of strings, but we're
+                    # expecting bytes
+                    b = ''.join(self._response._result.body)
+                else:
+                    raise
+
+            if isinstance(b, bytes) and bytes is not str:
+                b = b.decode("utf-8")
+            self._body = ''.join(b)
 
         return self._body
 
@@ -107,7 +127,7 @@ class IManagerSetup(zope.interface.Interface):
         """
 
 
-class BaseDatabaseFactory(object):
+class BaseDatabaseFactory:
     """Factory object for passing to appsetup.multi_databases
 
     This class is an internal implementation detail, subject to change
@@ -137,7 +157,7 @@ class BaseDatabaseFactory(object):
         return DB(storage, database_name=name)
 
 
-class DerivedDatabaseFactory(object):
+class DerivedDatabaseFactory:
     """Factory object for passing to appsetup.multi_databases
 
     This class is an internal implementation detail, subject to change
@@ -162,7 +182,7 @@ class DerivedDatabaseFactory(object):
         return DB(self.storage, database_name=self.name)
 
 
-class FunctionalTestSetup(object):
+class FunctionalTestSetup:
     """Keeps shared state across several functional test cases."""
 
     __shared_state = {'_init': False}
@@ -188,7 +208,7 @@ class FunctionalTestSetup(object):
                 config_file = 'ftesting.zcml'
             if database_names is None:
                 database_names = ('unnamed',)
-            self.log = StringIO()
+            self.log = io.StringIO()
             # Make it silent but keep the log available for debugging
             logging.root.addHandler(logging.StreamHandler(self.log))
 
@@ -197,7 +217,7 @@ class FunctionalTestSetup(object):
             configs = []
             if product_config:
                 configs = zope.app.appsetup.product.loadConfiguration(
-                    StringIO(product_config))
+                    io.StringIO(product_config))
                 configs = [
                     zope.app.appsetup.product.FauxConfiguration(name, values)
                     for name, values in configs.items()]
@@ -356,7 +376,8 @@ def defineLayer(name, zcml='test.zcml', allow_teardown=False):
         globals['__name__'],
         name,
         allow_teardown=allow_teardown,
-        )
+    )
+
 
 if os.path.exists(os.path.join('zopeskel', 'etc', 'ftesting.zcml')):
     Functional = os.path.join('zopeskel', 'etc', 'ftesting.zcml')
@@ -387,13 +408,13 @@ class FunctionalTestCase(unittest.TestCase):
 
     def setUp(self):
         """Prepares for a functional test case."""
-        super(FunctionalTestCase, self).setUp()
+        super().setUp()
         FunctionalTestSetup().setUp()
 
     def tearDown(self):
         """Cleans up after a functional test case."""
         FunctionalTestSetup().tearDown()
-        super(FunctionalTestCase, self).tearDown()
+        super().tearDown()
 
     def getRootFolder(self):
         """Returns the Zope root folder."""
@@ -406,18 +427,18 @@ class FunctionalTestCase(unittest.TestCase):
         abort()
 
 
-class CookieHandler(object):
+class CookieHandler:
 
     def __init__(self, *args, **kw):
         # Somewhere to store cookies between consecutive requests
         self.cookies = SimpleCookie()
-        super(CookieHandler, self).__init__(*args, **kw)
+        super().__init__(*args, **kw)
 
     def httpCookie(self, path):
         """Return self.cookies as an HTTP_COOKIE environment value."""
-        l = [m.OutputString().split(';')[0] for m in self.cookies.values()
-             if path.startswith(m['path'])]
-        return '; '.join(l)
+        list_ = [m.OutputString().split(';')[0] for m in self.cookies.values()
+                 if path.startswith(m['path'])]
+        return '; '.join(list_)
 
     def loadCookies(self, envstring):
         self.cookies.load(envstring)
@@ -430,8 +451,10 @@ class CookieHandler(object):
         # cookies
         # TODO: handle cookie expirations
         for k, v in response._cookies.items():
-            k = k.encode('utf8')
-            self.cookies[k] = v['value'].encode('utf8')
+            k = k.encode('utf8') if bytes is str else k
+            val = v['value']
+            val = val.encode('utf8') if bytes is str else val
+            self.cookies[k] = val
             if 'path' in v:
                 self.cookies[k]['path'] = v['path']
 
@@ -517,18 +540,26 @@ class BrowserTestCase(CookieHandler, FunctionalTestCase):
         old_site = self.getSite()
         self.setSite(None)
 
-        from htmllib import HTMLParser
-        from formatter import NullFormatter
+        from html.parser import HTMLParser
 
         class SimpleHTMLParser(HTMLParser):
-            def __init__(self, fmt, base):
-                HTMLParser.__init__(self, fmt)
+            def __init__(self, base):
+                super().__init__()
                 self.base = base
+                self.anchorlist = []
 
             def do_base(self, attrs):
                 self.base = dict(attrs).get('href', self.base)
 
-        parser = SimpleHTMLParser(NullFormatter(), path)
+            def handle_starttag(self, tag, attrs):
+                if tag == 'a':
+                    attrs = dict(attrs)
+                    if 'href' in attrs:
+                        self.anchorlist.append(attrs['href'])
+
+        parser = SimpleHTMLParser(path)
+        if bytes is not str and not isinstance(body, str):
+            body = body.decode("utf-8")
         parser.feed(body)
         parser.close()
         base = parser.base
@@ -577,7 +608,8 @@ class BrowserTestCase(CookieHandler, FunctionalTestCase):
                     request.close()
         if errors:
             self.fail("%s contains broken links:\n" % path
-                      + "\n".join(["  %s:\t%s" % (a, e) for a, e in errors]))
+                      + "\n".join(
+                          ["  {}:\t{}".format(a, e) for a, e in errors]))
 
 
 class HTTPTestCase(FunctionalTestCase):
@@ -649,7 +681,12 @@ def auth_header(header):
             u = ''
         if p is None:
             p = ''
-        auth = base64.encodestring('%s:%s' % (u, p))
+        user_pass = '{}:{}'.format(u, p)
+        encoder = getattr(base64, 'encodebytes', None)
+        if encoder is None:
+            encoder = getattr(base64, 'encodestring')
+        auth = encoder(user_pass.encode("latin-1"))
+        auth = auth.decode('ascii')
         return 'Basic %s' % auth[:-1]
     return header
 
@@ -668,25 +705,26 @@ class SampleFunctionalTest(BrowserTestCase):
 
     def testRootPage(self):
         response = self.publish('/')
-        self.assertEquals(response.getStatus(), 200)
+        self.assertEqual(response.getStatus(), 200)
 
     def testRootPage_preferred_languages(self):
         response = self.publish('/', env={'HTTP_ACCEPT_LANGUAGE': 'en'})
-        self.assertEquals(response.getStatus(), 200)
+        self.assertEqual(response.getStatus(), 200)
 
     def testNotExisting(self):
         response = self.publish('/nosuchthing', handle_errors=True)
-        self.assertEquals(response.getStatus(), 404)
+        self.assertEqual(response.getStatus(), 404)
 
     def testLinks(self):
         response = self.publish('/')
-        self.assertEquals(response.getStatus(), 200)
+        self.assertEqual(response.getStatus(), 200)
         self.checkForBrokenLinks(response.consumeBody(), response.getPath())
 
 
 def sample_test_suite():
     suite = unittest.TestSuite()
-    suite.addTest(unittest.makeSuite(SampleFunctionalTest))
+    suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(
+        SampleFunctionalTest))
     return suite
 
 
@@ -701,21 +739,26 @@ class HTTPCaller(CookieHandler):
         request_string = request_string.lstrip()
 
         # split off and parse the command line
-        l = request_string.find('\n')
-        command_line = request_string[:l].rstrip()
-        request_string = request_string[l + 1:]
+        line = request_string.find('\n')
+        command_line = request_string[:line].rstrip()
+        request_string = request_string[line + 1:]
         method, path, protocol = command_line.split()
 
-        instream = StringIO(request_string)
-        environment = {"HTTP_COOKIE": self.httpCookie(path),
-                       "HTTP_HOST": 'localhost',
-                       "REQUEST_METHOD": method,
-                       "SERVER_PROTOCOL": protocol,
-                       "REMOTE_ADDR": '127.0.0.1',
-                       }
+        # If we don't feed bytes, it gets stuck in a loop
+        # and ultimately raises HTTPException: got more than 100 headers.
+        instream = io.BytesIO(request_string.encode("latin-1")
+                              if not isinstance(request_string, bytes)
+                              else request_string)
+        environment = {
+            "HTTP_COOKIE": self.httpCookie(path),
+            "HTTP_HOST": 'localhost',
+            "REQUEST_METHOD": method,
+            "SERVER_PROTOCOL": protocol,
+            "REMOTE_ADDR": '127.0.0.1',
+        }
 
         headers = [split_header(header)
-                   for header in rfc822.Message(instream).headers]
+                   for header in headers_factory(instream).headers]
         for name, value in headers:
             name = ('_'.join(name.upper().split('-')))
             if name not in ('CONTENT_TYPE', 'CONTENT_LENGTH'):
@@ -751,7 +794,7 @@ class HTTPCaller(CookieHandler):
         response = ResponseWrapper(
             request.response, path, request,
             omit=('x-content-type-warning', 'x-powered-by'),
-            )
+        )
 
         self.saveCookies(response)
         setSite(old_site)
@@ -790,6 +833,7 @@ def _prepare_doctest_keywords(kw):
     globs['sync'] = sync
 
     kwsetUp = kw.get('setUp')
+
     def setUp(test):
         test.globs['http'] = HTTPCaller()
         FunctionalTestSetup().setUp()
@@ -798,6 +842,7 @@ def _prepare_doctest_keywords(kw):
     kw['setUp'] = setUp
 
     kwtearDown = kw.get('tearDown')
+
     def tearDown(test):
         if kwtearDown is not None:
             kwtearDown(test)
