@@ -13,51 +13,71 @@
 ##############################################################################
 """Setting up an environment for testing context-dependent objects
 
-$Id: setup.py 110724 2010-04-11 00:04:11Z tseaver $
 """
-import zope.component
-import zope.traversing.api
+import sys
 
-#------------------------------------------------------------------------
+import zope.component
+import zope.component.hooks
+import zope.component.interfaces
+import zope.traversing.api
+# ------------------------------------------------------------------------
 # Annotations
 from zope.annotation.attribute import AttributeAnnotations
-def setUpAnnotations():
-    zope.component.provideAdapter(AttributeAnnotations)
-
-#------------------------------------------------------------------------
-# Dependencies
 from zope.annotation.interfaces import IAttributeAnnotatable
 from zope.app.dependable import Dependable
 from zope.app.dependable.interfaces import IDependable
+from zope.container.interfaces import ISimpleReadContainer
+from zope.container.traversal import ContainerTraversable
+from zope.interface import Interface
+from zope.interface.interfaces import IComponentLookup
+from zope.site.folder import Folder
+from zope.site.folder import rootFolder
+from zope.site.site import LocalSiteManager
+from zope.site.site import SiteManagerAdapter
+from zope.testing.module import FakeModule
+from zope.traversing.interfaces import ITraversable
+
+from zope.app.testing.placelesssetup import setUp as placelessSetUp
+from zope.app.testing.placelesssetup import tearDown as placelessTearDown
+
+
+def setUpAnnotations():
+    zope.component.provideAdapter(AttributeAnnotations)
+
+
+# ------------------------------------------------------------------------
+# Dependencies
+
+
 def setUpDependable():
     zope.component.provideAdapter(Dependable, (IAttributeAnnotatable,),
                                   IDependable)
 
-#------------------------------------------------------------------------
+
+# ------------------------------------------------------------------------
 # Traversal
-from zope.traversing.interfaces import ITraversable
-from zope.container.interfaces import ISimpleReadContainer
-from zope.container.traversal import ContainerTraversable
+
+
 def setUpTraversal():
     from zope.traversing.testing import setUp
     setUp()
     zope.component.provideAdapter(ContainerTraversable,
                                   (ISimpleReadContainer,), ITraversable)
 
-#------------------------------------------------------------------------
+
+# ------------------------------------------------------------------------
 # ISiteManager lookup
-from zope.site.site import SiteManagerAdapter
-from zope.component.interfaces import IComponentLookup
-from zope.interface import Interface
+
+
 def setUpSiteManagerLookup():
     zope.component.provideAdapter(SiteManagerAdapter, (Interface,),
                                   IComponentLookup)
 
-#------------------------------------------------------------------------
+
+# ------------------------------------------------------------------------
 # Placeful setup
-import zope.component.hooks
-from zope.app.testing.placelesssetup import setUp as placelessSetUp
-from zope.app.testing.placelesssetup import tearDown as placelessTearDown
+
+
 def placefulSetUp(site=False):
     placelessSetUp()
     zope.component.hooks.setHooks()
@@ -71,14 +91,17 @@ def placefulSetUp(site=False):
         createSiteManager(site, setsite=True)
         return site
 
+
 def placefulTearDown():
     placelessTearDown()
     zope.component.hooks.resetHooks()
     zope.component.hooks.setSite()
 
-#------------------------------------------------------------------------
+
+# ------------------------------------------------------------------------
 # Sample Folder Creation
-from zope.site.folder import Folder, rootFolder
+
+
 def buildSampleFolderTree():
     # set up a reasonably complex folder structure
     #
@@ -91,38 +114,37 @@ def buildSampleFolderTree():
     # folder1_1_1 folder1_1_2  folder1_2_1  folder2_1_1
 
     root = rootFolder()
-    root[u'folder1'] = Folder()
-    root[u'folder1'][u'folder1_1'] = Folder()
-    root[u'folder1'][u'folder1_1'][u'folder1_1_1'] = Folder()
-    root[u'folder1'][u'folder1_1'][u'folder1_1_2'] = Folder()
-    root[u'folder1'][u'folder1_2'] = Folder()
-    root[u'folder1'][u'folder1_2'][u'folder1_2_1'] = Folder()
-    root[u'folder2'] = Folder()
-    root[u'folder2'][u'folder2_1'] = Folder()
-    root[u'folder2'][u'folder2_1'][u'folder2_1_1'] = Folder()
-    root[u"\N{CYRILLIC SMALL LETTER PE}"
-         u"\N{CYRILLIC SMALL LETTER A}"
-         u"\N{CYRILLIC SMALL LETTER PE}"
-         u"\N{CYRILLIC SMALL LETTER KA}"
-         u"\N{CYRILLIC SMALL LETTER A}3"] = Folder()
-    root[u"\N{CYRILLIC SMALL LETTER PE}"
-         u"\N{CYRILLIC SMALL LETTER A}"
-         u"\N{CYRILLIC SMALL LETTER PE}"
-         u"\N{CYRILLIC SMALL LETTER KA}"
-         u"\N{CYRILLIC SMALL LETTER A}3"][
-         u"\N{CYRILLIC SMALL LETTER PE}"
-         u"\N{CYRILLIC SMALL LETTER A}"
-         u"\N{CYRILLIC SMALL LETTER PE}"
-         u"\N{CYRILLIC SMALL LETTER KA}"
-         u"\N{CYRILLIC SMALL LETTER A}3_1"] = Folder()
+    root['folder1'] = Folder()
+    root['folder1']['folder1_1'] = Folder()
+    root['folder1']['folder1_1']['folder1_1_1'] = Folder()
+    root['folder1']['folder1_1']['folder1_1_2'] = Folder()
+    root['folder1']['folder1_2'] = Folder()
+    root['folder1']['folder1_2']['folder1_2_1'] = Folder()
+    root['folder2'] = Folder()
+    root['folder2']['folder2_1'] = Folder()
+    root['folder2']['folder2_1']['folder2_1_1'] = Folder()
+    root["\N{CYRILLIC SMALL LETTER PE}"
+         "\N{CYRILLIC SMALL LETTER A}"
+         "\N{CYRILLIC SMALL LETTER PE}"
+         "\N{CYRILLIC SMALL LETTER KA}"
+         "\N{CYRILLIC SMALL LETTER A}3"] = Folder()
+    root["\N{CYRILLIC SMALL LETTER PE}"
+         "\N{CYRILLIC SMALL LETTER A}"
+         "\N{CYRILLIC SMALL LETTER PE}"
+         "\N{CYRILLIC SMALL LETTER KA}"
+         "\N{CYRILLIC SMALL LETTER A}3"][
+        "\N{CYRILLIC SMALL LETTER PE}"
+        "\N{CYRILLIC SMALL LETTER A}"
+        "\N{CYRILLIC SMALL LETTER PE}"
+        "\N{CYRILLIC SMALL LETTER KA}"
+        "\N{CYRILLIC SMALL LETTER A}3_1"] = Folder()
 
     return root
 
 
-#------------------------------------------------------------------------
+# ------------------------------------------------------------------------
 # Sample Folder Creation
-from zope.site.site import LocalSiteManager
-import zope.component.interfaces
+
 
 def createSiteManager(folder, setsite=False):
     if not zope.component.interfaces.ISite.providedBy(folder):
@@ -132,7 +154,7 @@ def createSiteManager(folder, setsite=False):
     return zope.traversing.api.traverse(folder, "++etc++site")
 
 
-#------------------------------------------------------------------------
+# ------------------------------------------------------------------------
 # Local Utility Addition
 def addUtility(sitemanager, name, iface, utility, suffix=''):
     """Add a utility to a site manager
@@ -147,36 +169,24 @@ def addUtility(sitemanager, name, iface, utility, suffix=''):
     return utility
 
 
-#------------------------------------------------------------------------
+# ------------------------------------------------------------------------
 # Setup of test text files as modules
-import sys
 
 # Evil hack to make pickling work with classes defined in doc tests
+
 class NoCopyDict(dict):
     def copy(self):
         return self
 
-class FakeModule:
-    """A fake module."""
-    
-    def __init__(self, dict):
-        self.__dict = dict
-
-    def __getattr__(self, name):
-        try:
-            return self.__dict[name]
-        except KeyError:
-            raise AttributeError(name)
-
 
 def setUpTestAsModule(test, name=None):
     if name is None:
-        if test.globs.has_key('__name__'):
+        if '__name__' in test.globs:
             name = test.globs['__name__']
         else:
             name = test.globs.name
 
-    test.globs['__name__'] = name 
+    test.globs['__name__'] = name
     test.globs = NoCopyDict(test.globs)
     sys.modules[name] = FakeModule(test.globs)
 
@@ -184,4 +194,3 @@ def setUpTestAsModule(test, name=None):
 def tearDownTestAsModule(test):
     del sys.modules[test.globs['__name__']]
     test.globs.clear()
-
